@@ -1,14 +1,14 @@
 package Monitoring::Availability;
 
-use 5.006;
+use 5.008;
 use strict;
 use warnings;
 use Data::Dumper;
 use Carp;
-use POSIX qw(strftime);
+use POSIX qw(strftime mktime);
 use Monitoring::Availability::Logs;
 
-our $VERSION = '0.11_1';
+our $VERSION = '0.27_1';
 
 
 =head1 NAME
@@ -88,6 +88,12 @@ Time format for the log output, default: %s
 
 verbose mode
 
+=item breakdown
+
+Breakdown availability into 'months', 'weeks', 'days', 'none'
+
+adds additional 'breakdown' hash to each result with broken down results
+
 =back
 
 =cut
@@ -116,6 +122,11 @@ use constant {
 
     HOST_ONLY           => 2,
     SERVICE_ONLY        => 3,
+
+    BREAK_NONE          => 0,
+    BREAK_DAYS          => 1,
+    BREAK_WEEKS         => 2,
+    BREAK_MONTHS        => 3,
 };
 
 sub new {
@@ -135,6 +146,7 @@ sub new {
         'initialassumedservicestate'     => undef,
         'backtrack'                      => undef,
         'showscheduleddowntime'          => undef,
+        'breakdown'                      => undef,
     };
 
     bless $self, $class;
@@ -265,7 +277,7 @@ sub calculate {
         'log_livestatus'                 => undef,   # logs from a livestatus query
         'log_file'                       => undef,   # logs from a file
         'log_dir'                        => undef,   # logs from a dir
-        'rpttimeperiod'                  => $self->{'rpttimeperiod'},
+        'rpttimeperiod'                  => $self->{'rpttimeperiod'} || '',
         'assumeinitialstates'            => $self->{'assumeinitialstates'},
         'assumestateretention'           => $self->{'assumestateretention'},
         'assumestatesduringnotrunning'   => $self->{'assumestatesduringnotrunning'},
@@ -275,6 +287,7 @@ sub calculate {
         'backtrack'                      => $self->{'backtrack'},
         'showscheduleddowntime'          => $self->{'showscheduleddowntime'},
         'timeformat'                     => $self->{'timeformat'},
+        'breakdown'                      => $self->{'breakdown'},
     };
     $self->_log('calculate()') if $self->{'verbose'};
     my $result;
@@ -329,6 +342,8 @@ sub calculate {
         $self->_set_empty_services($result);
     }
 
+    $self->_set_breakpoints();
+
     # read in logs
     if(defined $self->{'report_options'}->{'log_string'} or $self->{'report_options'}->{'log_file'} or $self->{'report_options'}->{'log_dir'}) {
         my $mal = Monitoring::Availability::Logs->new(
@@ -428,19 +443,7 @@ sub _set_empty_hosts {
             if($@) { croak("found no initial state for host '$hostname'\ngot: ".Dumper($self->{'report_options'}->{'initial_states'}).Dumper($@)); }
             $self->{'report_options'}->{'first_state'} = $first_state;
         }
-        $data->{'hosts'}->{$hostname} = {
-            'time_up'           => 0,
-            'time_down'         => 0,
-            'time_unreachable'  => 0,
-
-            'scheduled_time_up'             => 0,
-            'scheduled_time_down'           => 0,
-            'scheduled_time_unreachable'    => 0,
-            'scheduled_time_indeterminate'  => 0,
-
-            'time_indeterminate_nodata'     => 0,
-            'time_indeterminate_notrunning' => 0,
-        };
+        $data->{'hosts'}->{$hostname} = $self->_new_host_data($self->{'report_options'}->{'breakdown'});
         $self->{'host_data'}->{$hostname} = {
             'in_downtime'      => 0,
             'last_state'       => $initial_assumend_state,
@@ -472,21 +475,7 @@ sub _set_empty_services {
                 if($@) { croak("found no initial state for service '$service_description' on host '$hostname'\ngot: ".Dumper($self->{'report_options'}->{'initial_states'}).Dumper($@)); }
                 $self->{'report_options'}->{'first_state'} = $first_state;
             }
-            $data->{'services'}->{$hostname}->{$service_description} = {
-                'time_ok'           => 0,
-                'time_warning'      => 0,
-                'time_unknown'      => 0,
-                'time_critical'     => 0,
-
-                'scheduled_time_ok'             => 0,
-                'scheduled_time_warning'        => 0,
-                'scheduled_time_unknown'        => 0,
-                'scheduled_time_critical'       => 0,
-                'scheduled_time_indeterminate'  => 0,
-
-                'time_indeterminate_nodata'     => 0,
-                'time_indeterminate_notrunning' => 0,
-            };
+            $data->{'services'}->{$hostname}->{$service_description} = $self->_new_service_data($self->{'report_options'}->{'breakdown'});
 
             # create last service data
             $self->{'service_data'}->{$hostname}->{$service_description} = {
@@ -520,12 +509,20 @@ sub _compute_for_data {
 
     # if we reach the start date of our report, insert a fake entry
     if($last_time < $self->{'report_options'}->{'start'} and $data->{'time'} > $self->{'report_options'}->{'start'}) {
-        $self->_insert_fake_start_event($result);
+        $self->_insert_fake_event($result, $self->{'report_options'}->{'start'});
+    }
+
+    # if we passed a breakdown point, insert fake event
+    if($self->{'report_options'}->{'breakdown'} != BREAK_NONE) {
+        my $breakpoint = $self->{'breakpoints'}->[0];
+        if(defined $breakpoint and $last_time < $breakpoint and $data->{'time'} > $breakpoint) {
+            $self->_insert_fake_event($result, $breakpoint);
+        }
     }
 
     # end of report reached, insert fake end event
-    if($data->{'time'} > $self->{'report_options'}->{'end'} and $last_time < $self->{'report_options'}->{'end'}) {
-        $self->_insert_fake_end_event($result);
+    if($data->{'time'} >= $self->{'report_options'}->{'end'} and $last_time < $self->{'report_options'}->{'end'}) {
+        $self->_insert_fake_event($result, $self->{'report_options'}->{'end'});
 
         # set a log entry
         $self->_add_log_entry(
@@ -618,12 +615,12 @@ sub _add_last_time_event {
 
     # no start event yet, insert a fake entry
     if($last_time < $self->{'report_options'}->{'start'}) {
-        $self->_insert_fake_start_event($result);
+        $self->_insert_fake_event($result, $self->{'report_options'}->{'start'});
     }
 
     # no end event yet, insert fake end event
     if($last_time < $self->{'report_options'}->{'end'}) {
-        $self->_insert_fake_end_event($result);
+        $self->_insert_fake_event($result, $self->{'report_options'}->{'end'});
     }
 
     return 1;
@@ -647,24 +644,6 @@ sub _process_log_line {
         return;
     }
 
-    # skip hosts we dont need
-    if($self->{'report_options'}->{'calc_all'} == 0 and defined $data->{'host_name'} and !defined $self->{'host_data'}->{$data->{'host_name'}} and !defined $self->{'service_data'}->{$data->{'host_name'}}) {
-        $self->_log('  -> skipped not needed host event') if $self->{'verbose'};
-        return;
-    }
-
-    # skip services we dont need
-    if($self->{'report_options'}->{'calc_all'} == 0
-       and defined $data->{'host_name'}
-       and defined $data->{'service_description'}
-       and $data->{'service_description'} ne ''
-       and !defined $self->{'service_data'}->{$data->{'host_name'}}->{$data->{'service_description'}}
-      ) {
-        $self->_log('  -> skipped not needed service event') if $self->{'verbose'};
-        return;
-    }
-
-
     # process starts / stops?
     if(defined $data->{'proc_start'}) {
         unless($self->{'report_options'}->{'assumestatesduringnotrunning'}) {
@@ -727,8 +706,64 @@ sub _process_log_line {
         }
     }
 
+    # timeperiod transitions
+    elsif(    defined $data->{'timeperiod'} ) {
+        if($self->{'report_options'}->{'rpttimeperiod'} eq $data->{'timeperiod'} ) {
+            $self->_log('_process_log_line() timeperiod translation, inserting fake event for all hosts/services') if $self->{'verbose'};
+            for my $host_name (keys %{$self->{'service_data'}}) {
+                for my $service_description (keys %{$self->{'service_data'}->{$host_name}}) {
+                    my $last_known_state = $self->{'service_data'}->{$host_name}->{$service_description}->{'last_known_state'};
+                    my $last_state = STATE_UNSPECIFIED;
+                    $last_state = $last_known_state if defined $last_known_state and $last_known_state >= 0;
+                    $self->_set_service_event($host_name, $service_description, $result, { 'start' => $data->{'start'}, 'end' => $data->{'end'}, 'time' => $data->{'time'}, 'state' => $last_state });
+                }
+            }
+            for my $host_name (keys %{$self->{'host_data'}}) {
+                my $last_known_state = $self->{'host_data'}->{$host_name}->{'last_known_state'};
+                my $last_state = STATE_UNSPECIFIED;
+                $last_state = $last_known_state if defined $last_known_state and $last_known_state >= 0;
+                $self->_set_host_event($host_name, $result, { 'time' => $data->{'time'}, 'state' => $last_state });
+            }
+            $self->{'in_timeperiod'} = $data->{'to'};
+
+            # set a log entry
+            my $start         = 'STOP';
+            my $plugin_output = 'leaving timeperiod: '.$data->{'timeperiod'};
+            if($self->{'in_timeperiod'}) {
+                $plugin_output = 'entering timeperiod: '.$data->{'timeperiod'};
+                $start         = 'START';
+            }
+            $self->_add_log_entry(
+                            'full_only'   => 1,
+                            'log'         => {
+                                'start'         => $data->{'time'},
+                                'type'          => 'TIMEPERIOD '.$start,
+                                'plugin_output' => $plugin_output,
+                                'class'         => 'INDETERMINATE',
+                            },
+            );
+        }
+    }
+
+    # skip hosts we dont need
+    if($self->{'report_options'}->{'calc_all'} == 0 and defined $data->{'host_name'} and !defined $self->{'host_data'}->{$data->{'host_name'}} and !defined $self->{'service_data'}->{$data->{'host_name'}}) {
+        $self->_log('  -> skipped not needed host event') if $self->{'verbose'};
+        return;
+    }
+
+    # skip services we dont need
+    if($self->{'report_options'}->{'calc_all'} == 0
+       and defined $data->{'host_name'}
+       and defined $data->{'service_description'}
+       and $data->{'service_description'} ne ''
+       and !defined $self->{'service_data'}->{$data->{'host_name'}}->{$data->{'service_description'}}
+      ) {
+        $self->_log('  -> skipped not needed service event') if $self->{'verbose'};
+        return;
+    }
+
     # service events
-    elsif(defined $data->{'service_description'} and $data->{'service_description'} ne '') {
+    if(defined $data->{'service_description'} and $data->{'service_description'} ne '') {
         my $service_hist = $self->{'service_data'}->{$data->{'host_name'}}->{$data->{'service_description'}};
 
         if($data->{'type'} eq 'CURRENT SERVICE STATE' or $data->{'type'} eq 'SERVICE ALERT' or $data->{'type'} eq 'INITIAL SERVICE STATE') {
@@ -817,7 +852,7 @@ sub _process_log_line {
 
             my $last_state_time = $host_hist->{'last_state_time'};
 
-            $self->_log('_process_log_line() hostdowntime, inserting fake event for all services') if $self->{'verbose'};
+            $self->_log('_process_log_line() hostdowntime, inserting fake event for all hosts/services') if $self->{'verbose'};
             # set an event for all services
             for my $service_description (keys %{$self->{'service_data'}->{$data->{'host_name'}}}) {
                 $last_state_time = $self->{'service_data'}->{$data->{'host_name'}}->{$service_description}->{'last_state_time'};
@@ -884,60 +919,39 @@ sub _set_service_event {
         if(defined $service_hist->{'last_state'}) {
             my $diff = $data->{'time'} - $service_hist->{'last_state_time'};
 
+            # outside timeperiod
+            if(defined $self->{'in_timeperiod'} and !$self->{'in_timeperiod'}) {
+                $self->_add_time($service_data, $data->{'time'}, 'time_indeterminate_outside_timeperiod', $diff);
+            }
+
             # ok
-            if($service_hist->{'last_state'} == STATE_OK) {
-                $self->_log('_set_service_event() ok + '.$diff.' seconds ('.$self->_duration($diff).')') if $self->{'verbose'};
-                $service_data->{'time_ok'} += $diff;
-                if($service_hist->{'in_downtime'} or $host_hist->{'in_downtime'}) {
-                    $self->_log('_set_service_event() ok sched + '.$diff.' seconds') if $self->{'verbose'};
-                    $service_data->{'scheduled_time_ok'} += $diff
-                }
+            elsif($service_hist->{'last_state'} == STATE_OK) {
+                $self->_add_time($service_data, $data->{'time'}, 'time_ok', $diff, ($service_hist->{'in_downtime'} or $host_hist->{'in_downtime'}));
             }
 
             # warning
             elsif($service_hist->{'last_state'} == STATE_WARNING) {
-                $self->_log('_set_service_event() warning + '.$diff.' seconds') if $self->{'verbose'};
-                $service_data->{'time_warning'} += $diff;
-                if($service_hist->{'in_downtime'} or $host_hist->{'in_downtime'}) {
-                    $self->_log('_set_service_event() warning sched + '.$diff.' seconds') if $self->{'verbose'};
-                    $service_data->{'scheduled_time_warning'} += $diff
-                }
+                $self->_add_time($service_data, $data->{'time'}, 'time_warning', $diff, ($service_hist->{'in_downtime'} or $host_hist->{'in_downtime'}));
             }
 
             # critical
             elsif($service_hist->{'last_state'} == STATE_CRITICAL) {
-                $self->_log('_set_service_event() critical + '.$diff.' seconds ('.$self->_duration($diff).')') if $self->{'verbose'};
-                $service_data->{'time_critical'} += $diff;
-                if($service_hist->{'in_downtime'} or $host_hist->{'in_downtime'}) {
-                    $self->_log('_set_service_event() critical sched + '.$diff.' seconds') if $self->{'verbose'};
-                    $service_data->{'scheduled_time_critical'} += $diff
-                }
+                $self->_add_time($service_data, $data->{'time'}, 'time_critical', $diff, ($service_hist->{'in_downtime'} or $host_hist->{'in_downtime'}));
             }
 
             # unknown
             elsif($service_hist->{'last_state'} == STATE_UNKNOWN) {
-                $self->_log('_set_service_event() unknown + '.$diff.' seconds ('.$self->_duration($diff).')') if $self->{'verbose'};
-                $service_data->{'time_unknown'} += $diff;
-                if($service_hist->{'in_downtime'} or $host_hist->{'in_downtime'}) {
-                    $self->_log('_set_service_event() unknown sched + '.$diff.' seconds') if $self->{'verbose'};
-                    $service_data->{'scheduled_time_unknown'} += $diff
-                }
+                $self->_add_time($service_data, $data->{'time'}, 'time_unknown', $diff, ($service_hist->{'in_downtime'} or $host_hist->{'in_downtime'}));
             }
 
             # no data yet
             elsif($service_hist->{'last_state'} == STATE_UNSPECIFIED) {
-                $self->_log('_set_service_event() indeterminate + '.$diff.' seconds ('.$self->_duration($diff).')') if $self->{'verbose'};
-                $service_data->{'time_indeterminate_nodata'} += $diff;
-                if($service_hist->{'in_downtime'} or $host_hist->{'in_downtime'}) {
-                    $self->_log('_set_service_event() indeterminate sched + '.$diff.' seconds') if $self->{'verbose'};
-                    $service_data->{'scheduled_time_indeterminate'} += $diff
-                }
+                $self->_add_time($service_data, $data->{'time'}, 'time_indeterminate_nodata', $diff, ($service_hist->{'in_downtime'} or $host_hist->{'in_downtime'}), 'scheduled_time_indeterminate');
             }
 
             # not running
             elsif($service_hist->{'last_state'} == STATE_NOT_RUNNING) {
-                $self->_log('_set_service_event() not_running + '.$diff.' seconds ('.$self->_duration($diff).')') if $self->{'verbose'};
-                $service_data->{'time_indeterminate_notrunning'} += $diff;
+                $self->_add_time($service_data, $data->{'time'}, 'time_indeterminate_notrunning', $diff);
             }
 
         }
@@ -975,50 +989,34 @@ sub _set_host_event {
         if(defined $host_hist->{'last_state'}) {
             my $diff = $data->{'time'} - $host_hist->{'last_state_time'};
 
+            # outside timeperiod
+            if(defined $self->{'in_timeperiod'} and !$self->{'in_timeperiod'}) {
+                $self->_add_time($host_data, $data->{'time'}, 'time_indeterminate_outside_timeperiod', $diff);
+            }
+
             # up
-            if($host_hist->{'last_state'} == STATE_UP) {
-                $self->_log('_set_host_event() up + '.$diff.' seconds ('.$self->_duration($diff).')') if $self->{'verbose'};
-                $host_data->{'time_up'} += $diff;
-                if($host_hist->{'in_downtime'}) {
-                    $self->_log('_set_host_event() up sched + '.$diff.' seconds') if $self->{'verbose'};
-                    $host_data->{'scheduled_time_up'} += $diff
-                }
+            elsif($host_hist->{'last_state'} == STATE_UP) {
+                $self->_add_time($host_data, $data->{'time'}, 'time_up', $diff, $host_hist->{'in_downtime'});
             }
 
             # down
             elsif($host_hist->{'last_state'} == STATE_DOWN) {
-                $self->_log('_set_host_event() down + '.$diff.' seconds') if $self->{'verbose'};
-                $host_data->{'time_down'} += $diff;
-                if($host_hist->{'in_downtime'}) {
-                    $self->_log('_set_host_event() down sched + '.$diff.' seconds') if $self->{'verbose'};
-                    $host_data->{'scheduled_time_down'} += $diff
-                }
+                $self->_add_time($host_data, $data->{'time'}, 'time_down', $diff, $host_hist->{'in_downtime'});
             }
 
             # unreachable
             elsif($host_hist->{'last_state'} == STATE_UNREACHABLE) {
-                $self->_log('_set_host_event() unreachable + '.$diff.' seconds ('.$self->_duration($diff).')') if $self->{'verbose'};
-                $host_data->{'time_unreachable'} += $diff;
-                if($host_hist->{'in_downtime'}) {
-                    $self->_log('_set_host_event() unreachable sched + '.$diff.' seconds') if $self->{'verbose'};
-                    $host_data->{'scheduled_time_unreachable'} += $diff
-                }
+                $self->_add_time($host_data, $data->{'time'}, 'time_unreachable', $diff, $host_hist->{'in_downtime'});
             }
 
             # no data yet
             elsif($host_hist->{'last_state'} == STATE_UNSPECIFIED) {
-                $self->_log('_set_host_event() indeterminate + '.$diff.' seconds ('.$self->_duration($diff).')') if $self->{'verbose'};
-                $host_data->{'time_indeterminate_nodata'} += $diff;
-                if($host_hist->{'in_downtime'}) {
-                    $self->_log('_set_host_event() indeterminate sched + '.$diff.' seconds') if $self->{'verbose'};
-                    $host_data->{'scheduled_time_indeterminate'} += $diff
-                }
+                $self->_add_time($host_data, $data->{'time'}, 'time_indeterminate_nodata', $diff, $host_hist->{'in_downtime'}, 'scheduled_time_indeterminate');
             }
 
             # not running
             elsif($host_hist->{'last_state'} == STATE_NOT_RUNNING) {
-                $self->_log('_set_host_event() not_running + '.$diff.' seconds ('.$self->_duration($diff).')') if $self->{'verbose'};
-                $host_data->{'time_indeterminate_notrunning'} += $diff;
+                $self->_add_time($host_data, $data->{'time'}, 'time_indeterminate_notrunning', $diff);
             }
         }
     }
@@ -1035,6 +1033,29 @@ sub _set_host_event {
     return 1;
 }
 
+########################################
+sub _add_time {
+    my($self, $data, $date, $type, $diff, $in_downtime, $scheduled_type) = @_;
+    $scheduled_type = 'scheduled_'.$type unless defined $scheduled_type;
+    $self->_log('_add_time() '.$type.' + '.$diff.' seconds ('.$self->_duration($diff).')') if $self->{'verbose'};
+    $data->{$type} += $diff;
+    if($in_downtime) {
+        $self->_log('_add_time() '.$type.' sched + '.$diff.' seconds') if $self->{'verbose'};
+        $data->{$scheduled_type} += $diff;
+    }
+
+    # breakdowns?
+    if($self->{'report_options'}->{'breakdown'} != BREAK_NONE) {
+        my($fmt, $timespan) = $self->_get_break_config();
+        my $timestr = strftime($fmt, localtime($date-1));
+        $data->{'breakdown'}->{$timestr}->{$type} += $diff;
+        if($in_downtime) {
+            $data->{'breakdown'}->{$timestr}->{$scheduled_type} += $diff;
+        }
+    }
+    return;
+}
+
 
 ########################################
 sub _log {
@@ -1085,11 +1106,12 @@ sub _duration {
 }
 
 ########################################
-sub _insert_fake_start_event {
+sub _insert_fake_event {
     my $self    = shift;
     my $result  = shift;
+    my $time    = shift;
 
-    $self->_log('_insert_fake_start_event()') if $self->{'verbose'};
+    $self->_log('_insert_fake_event()') if $self->{'verbose'};
     for my $host (keys %{$result->{'services'}}) {
         for my $service (keys %{$result->{'services'}->{$host}}) {
             my $last_service_state = STATE_UNSPECIFIED;
@@ -1098,7 +1120,7 @@ sub _insert_fake_start_event {
             }
             my $fakedata = {
                 'service_description' => $service,
-                'time'                => $self->{'report_options'}->{'start'},
+                'time'                => $time,
                 'host_name'           => $host,
                 'type'                => 'INITIAL SERVICE STATE',
                 'hard'                => 1,
@@ -1114,7 +1136,7 @@ sub _insert_fake_start_event {
             $last_host_state = $self->{'host_data'}->{$host}->{'last_state'};
         }
         my $fakedata = {
-            'time'                => $self->{'report_options'}->{'start'},
+            'time'                => $time,
             'host_name'           => $host,
             'type'                => 'INITIAL HOST STATE',
             'hard'                => 1,
@@ -1126,41 +1148,6 @@ sub _insert_fake_start_event {
     return 1;
 }
 
-########################################
-sub _insert_fake_end_event {
-    my $self    = shift;
-    my $result  = shift;
-
-    # process a fake last entry with our last known state
-    $self->_log('_insert_fake_end_event()') if $self->{'verbose'};
-    for my $host (keys %{$result->{'services'}}) {
-        for my $service (keys %{$result->{'services'}->{$host}}) {
-            my $fakedata = {
-                'service_description' => $service,
-                'time'                => $self->{'report_options'}->{'end'},
-                'host_name'           => $host,
-                'type'                => 'INITIAL SERVICE STATE',
-                'hard'                => 1,
-                'state'               => $self->{'service_data'}->{$host}->{$service}->{'last_state'},
-            };
-            $self->_set_service_event($host, $service, $result, $fakedata);
-        }
-    }
-
-    for my $host (keys %{$result->{'hosts'}}) {
-        my $fakedata = {
-            'time'                => $self->{'report_options'}->{'end'},
-            'host_name'           => $host,
-            'type'                => 'INITIAL HOST STATE',
-            'hard'                => 1,
-            'state'               => $self->{'host_data'}->{$host}->{'last_state'},
-        };
-        $self->_set_host_event($host, $result, $fakedata);
-    }
-
-    return 1;
-}
-
 ########################################
 sub _set_default_options {
     my $self    = shift;
@@ -1175,6 +1162,7 @@ sub _set_default_options {
     $options->{'initialassumedservicestate'}     = 'unspecified' unless defined $options->{'initialassumedservicestate'};
     $options->{'showscheduleddowntime'}          = 'yes'         unless defined $options->{'showscheduleddowntime'};
     $options->{'timeformat'}                     = '%s'          unless defined $options->{'timeformat'};
+    $options->{'breakdown'}                      = BREAK_NONE    unless defined $options->{'breakdown'};
 
     return $options;
 }
@@ -1257,6 +1245,31 @@ sub _verify_options {
         }
     }
 
+    # set breakdown
+    if(defined $options->{'breakdown'}) {
+        if(lc $options->{'breakdown'} eq 'months') {
+            $options->{'breakdown'} = BREAK_MONTHS;
+        }
+        elsif(lc $options->{'breakdown'} eq 'weeks') {
+            $options->{'breakdown'} = BREAK_WEEKS;
+        }
+        elsif(lc $options->{'breakdown'} eq 'days') {
+            $options->{'breakdown'} = BREAK_DAYS;
+        }
+        elsif(lc $options->{'breakdown'} eq 'none') {
+            $options->{'breakdown'} = BREAK_NONE;
+        }
+        elsif(   $options->{'breakdown'} == BREAK_NONE
+           or $options->{'breakdown'} == BREAK_DAYS
+           or $options->{'breakdown'} == BREAK_WEEKS
+           or $options->{'breakdown'} == BREAK_MONTHS) {
+            # ok
+        }
+        else {
+            croak('breakdown unknown, please use one of: months, weeks, days or none. Got: '.$options->{'breakdown'});
+        }
+    }
+
     return $options;
 }
 
@@ -1332,7 +1345,7 @@ sub _calculate_log {
         elsif($first_state == STATE_DOWN)        { $type = 'DOWN'; }
         elsif($first_state == STATE_UNREACHABLE) { $type = 'UNREACHABLE'; }
         my $fake_start = $self->{'report_options'}->{'start'};
-        if($fake_start >= $self->{'full_log_store'}->[0]->{'log'}->{'start'}) { $fake_start = $self->{'full_log_store'}->[0]->{'log'}->{'start'} - 1; }
+        if(defined $self->{'full_log_store'}->[0] and $fake_start >= $self->{'full_log_store'}->[0]->{'log'}->{'start'}) { $fake_start = $self->{'full_log_store'}->[0]->{'log'}->{'start'} - 1; }
         my $fakelog = {
             'log' => {
                 'type'          => 'HOST '.$type.' (HARD)',
@@ -1394,6 +1407,105 @@ sub _state_to_int {
     croak("valid values for services are: ok, warning, unknown and critical\nvalues for host: up, down and unreachable");
 }
 
+########################################
+sub _new_service_data {
+    my($self, $breakdown) = @_;
+    my $data = {
+        time_ok           => 0,
+        time_warning      => 0,
+        time_unknown      => 0,
+        time_critical     => 0,
+
+        scheduled_time_ok             => 0,
+        scheduled_time_warning        => 0,
+        scheduled_time_unknown        => 0,
+        scheduled_time_critical       => 0,
+        scheduled_time_indeterminate  => 0,
+
+        time_indeterminate_nodata             => 0,
+        time_indeterminate_notrunning         => 0,
+        time_indeterminate_outside_timeperiod => 0,
+    };
+    if($breakdown != BREAK_NONE) {
+        $data->{'breakdown'} = {};
+        my($fmt, $timespan) = $self->_get_break_config();
+        my $cur = $self->{'report_options'}->{'start'};
+        while($cur < $self->{'report_options'}->{'end'}) {
+            my $timestr = strftime($fmt, localtime($cur));
+            $data->{'breakdown'}->{$timestr} = $self->_new_service_data(BREAK_NONE);
+            $cur = $cur + $timespan;
+        }
+    }
+    return $data;
+}
+
+########################################
+sub _new_host_data {
+    my($self, $breakdown) = @_;
+    my $data = {
+        time_up           => 0,
+        time_down         => 0,
+        time_unreachable  => 0,
+
+        scheduled_time_up             => 0,
+        scheduled_time_down           => 0,
+        scheduled_time_unreachable    => 0,
+        scheduled_time_indeterminate  => 0,
+
+        time_indeterminate_nodata             => 0,
+        time_indeterminate_notrunning         => 0,
+        time_indeterminate_outside_timeperiod => 0,
+    };
+    if($breakdown != BREAK_NONE) {
+        $data->{'breakdown'} = {};
+        my($fmt, $timespan) = $self->_get_break_config();
+        my $cur = $self->{'report_options'}->{'start'};
+        while($cur < $self->{'report_options'}->{'end'}) {
+            my $timestr = strftime($fmt, localtime($cur));
+            $data->{'breakdown'}->{$timestr} = $self->_new_host_data(BREAK_NONE);
+            $cur = $cur + $timespan;
+        }
+    }
+    return $data;
+}
+
+########################################
+sub _get_break_config {
+    my($self) = @_;
+    my($fmt, $timespan);
+    if($self->{'report_options'}->{'breakdown'} == BREAK_DAYS) {
+        $fmt      = '%Y-%m-%d';
+        $timespan = 86400;
+    }
+    elsif($self->{'report_options'}->{'breakdown'} == BREAK_WEEKS) {
+        $fmt      = '%Y-KW%V';
+        $timespan = 86400 * 7;
+    }
+    elsif($self->{'report_options'}->{'breakdown'} == BREAK_MONTHS) {
+        $fmt      = '%Y-%m';
+        $timespan = 86400 * 30;
+    }
+    return($fmt, $timespan);
+}
+
+########################################
+sub _set_breakpoints {
+    my($self) = @_;
+    $self->{'breakpoints'} = [];
+
+    return if $self->{'report_options'}->{'breakdown'} == BREAK_NONE;
+
+    my $cur = $self->{'report_options'}->{'start'};
+    # round to next 0:00
+    my($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime($cur);
+    $cur = mktime(0, 0, 0, $mday, $mon, $year) + 86400;
+    while($cur < $self->{'report_options'}->{'end'}) {
+        push @{$self->{'breakpoints'}}, $cur;
+        $cur = $cur + 86400;
+    }
+    return;
+}
+
 ########################################
 
 1;
